<?php

/*******************************************************************************
 * Post As Another Member
 *
 * A modification for Simple Machines Forum 2.1 that adds the ability for a
 * member to post as another member.
 *
 * Copyright (c) 2023 Jon Stovell
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/

if (!defined('SMF'))
	die('No direct access...');

/**
 * Initializes some stuff we need and checks if the requested alterate user (if
 * any) actually exists
 *
 * Called by:
 * 		integrate_post_errors
 */
function postalt_post_errors(&$post_errors, &$minor_errors)
{
	global $context, $smcFunc, $board;

	loadLanguage('PostAsAnotherMember');

	$minor_errors[] = 'alternate_member';

	if (empty($board))
		$context['postalt_boards_allowed'] = boardsAllowedTo('post_as_alternative_user');
	elseif (allowedTo('post_as_alternative_user'))
		$context['postalt_boards_allowed'][] = $board;

	postalt_get_alt_id();
	$context['alternate_member'] = '';

	// Make sure the alternate user exists
	if (!empty($context['alternate_member_id']) && !empty($context['postalt_boards_allowed']))
	{
		$request = $smcFunc['db_query']('', '
			SELECT real_name
			FROM {db_prefix}members
			WHERE id_member = {int:alternate_id}',
			array(
				'alternate_id' => $context['alternate_member_id'],
			)
		);

		if ($smcFunc['db_num_rows']($request) == 1)
			list($context['alternate_member']) = $smcFunc['db_fetch_row']($request);
		else
			$post_errors[] = 'alternate_member';

		$smcFunc['db_free_result']($request);
	}
}

/**
 * Add some UI to the post authoring form
 *
 * Called by:
 * 		integrate_post_end
 */
function postalt_post_end()
{
	global $context, $txt;

	if (empty($context['postalt_boards_allowed']))
		return;

	loadJavaScriptFile('suggest.js', array('defer' => false), 'smf_suggest');

	$context['posting_fields']['alternate_member'] = array(
		'label' => array(
			'text' => !isset($_REQUEST['msg']) ? $txt['postalt_post_as'] : $txt['postalt_change'],
			'class' => isset($context['post_error']['alternate_member']) ? 'error' : '',
		),
		'input' => array(
			'type' => 'text',
			'attributes' => array(
				'id' => 'alternate_member',
				'value' => $context['alternate_member'],
				'size' => 80,
				'placeholder' => !isset($_REQUEST['msg']) ? $txt['postalt_post_as_desc'] : $txt['postalt_change_desc'],
			),
			'after' => '
			<div id="alternate_member_container" style="line-height:25px"></div>
			<script>
				var oMemberSuggest = new smc_AutoSuggest({
					sSelf: "oMemberSuggest",
					sSessionId: smf_session_id,
					sSessionVar: smf_session_var,
					sSuggestId: "alternate_member",
					sControlId: "alternate_member",
					sSearchType: "member",
					bItemList: true,
					sPostName: "alternate_member_list",
					sURLMask: "action=profile;u=%item_id%",
					sItemListContainerId: \'alternate_member_container\',
					aListItems: [' . (!empty($context['alternate_member']) && !empty($context['alternate_member_id']) ? '
						{
							sItemId: ' . $context['alternate_member_id'] . ',
							sItemName: ' . JavaScriptEscape($context['alternate_member']) . '
						}' : '') . '
					]
				});
				function postalt_afterAddItem(sSelf, newDivID, itemCount) {
					$("#alternate_member").hide();
				}
				function postalt_afterDeleteItem(sSelf, itemCount) {
					$("#alternate_member").show();
				}
				oMemberSuggest.registerCallback("onAfterAddItem", "postalt_afterAddItem");
				oMemberSuggest.registerCallback("onAfterDeleteItem", "postalt_afterDeleteItem");

				$(window).on("load", postalt_updateDisabledAttribute);
				$(".pf_board select[name=board]").on("change", postalt_updateDisabledAttribute);
				function postalt_updateDisabledAttribute(event) {
					var boards_allowed = ["' . implode('", "', $context['postalt_boards_allowed']) . '"];
					var selected_board = ' . (count($context['postalt_boards_allowed']) === 1 ? '"' . $context['postalt_boards_allowed'][0] . '"' : '$(".pf_board select[name=board]").val();') . '
					if (jQuery.inArray(selected_board, boards_allowed) !== -1) {
						$("#alternate_member").prop("disabled", false);
					}
					else {
						$("#alternate_member").prop("disabled", true);
						if ($("#alternate_member_container").html() !== "") {
							oMemberSuggest.deleteAddedItem(1);
						}
					}

				}
			</script>
			<noscript>' . $txt['postalt_enable_javascript'] . '</noscript>',
		),
	);
}

/**
 * Set an alternate user when creating a post
 *
 * Called by:
 * 		integrate_create_post
 */
function postalt_create_post(&$msgOptions, &$topicOptions, &$posterOptions, &$message_columns, &$message_parameters)
{
	global $context, $smcFunc;

	postalt_get_alt_id();

	if (!empty($context['alternate_member_id']) && allowedTo('post_as_alternative_user'))
	{
		$request = $smcFunc['db_query']('', '
			SELECT real_name, email_address
			FROM {db_prefix}members
			WHERE id_member = {int:alternate_id}',
			array(
				'alternate_id' => $context['alternate_member_id'],
			)
		);

		if ($smcFunc['db_num_rows']($request) == 1)
		{
			list($alt_name, $alt_email) = $smcFunc['db_fetch_row']($request);

			$posterOptions['id'] = $context['alternate_member_id'];
			$posterOptions['name'] = $alt_name;
			$posterOptions['email'] = $alt_email;

			$i = 0;
			foreach ($message_columns as $key => $value)
			{
				if ($key == 'id_member')
					$message_parameters[$i] = $context['alternate_member_id'];
				elseif ($key == 'poster_name')
					$message_parameters[$i] = $alt_name;
				elseif ($key == 'poster_email')
					$message_parameters[$i] = $alt_email;

				$i++;
			}
		}

		$smcFunc['db_free_result']($request);
	}
}

/**
 * Set an alternate user when modifying a post
 *
 * Called by:
 * 		integrate_modify_post
 */
function postalt_modify_post(&$messages_columns, &$update_parameters, &$msgOptions, &$topicOptions, &$posterOptions, &$messageInts)
{
	global $context, $smcFunc, $board_info;

	postalt_get_alt_id();

	if (!empty($context['alternate_member_id']) && allowedTo('post_as_alternative_user'))
	{
		$request = $smcFunc['db_query']('', '
			SELECT real_name, email_address
			FROM {db_prefix}members
			WHERE id_member = {int:alternate_id}',
			array(
				'alternate_id' => $context['alternate_member_id'],
			)
		);

		if ($smcFunc['db_num_rows']($request) == 1)
		{
			list($alt_name, $alt_email) = $smcFunc['db_fetch_row']($request);

			// Okay, the message was modified, but should we update post counts?
			if ($posterOptions['id'] != $context['alternate_member_id'] && $board_info['posts_count'])
			{
				updateMemberData($posterOptions['id'], array('posts' => '-'));
				updateMemberData($context['alternate_member_id'], array('posts' => '+'));
			}

			$posterOptions['id'] = $context['alternate_member_id'];
			$posterOptions['name'] = $alt_name;
			$posterOptions['email'] = $alt_email;

			$messages_columns['id_member'] = $context['alternate_member_id'];
			$messages_columns['poster_name'] = $alt_name;
			$messages_columns['poster_email'] = $alt_email;

			$messageInts[] = 'id_member';

			// Do this here, because modifyPost doesn't check for this possibility.
			$smcFunc['db_query']('', '
				UPDATE {db_prefix}topics
				SET id_member_started = {int:id_member}
				WHERE id_topic = {int:id_topic}
					AND id_first_msg = {int:id_message}',
				array(
					'id_member' => $context['alternate_member_id'],
					'id_topic' => $topicOptions['id'],
					'id_message' => $msgOptions['id'],
				)
			);
		}

		$smcFunc['db_free_result']($request);
	}
}

/**
 * Create the permission
 *
 * The permission is named 'post_as_alternative_user' to maintain compatibility
 * with an older mod for SMF 2.0
 *
 * Called by:
 * 		integrate_load_permissions
 */
function postalt_load_permissions(&$permissionGroups, &$permissionList, &$leftPermissionGroups, &$hiddenPermissions, &$relabelPermissions)
{
	loadLanguage('PostAsAnotherMember');

	$permissionList['board']['post_as_alternative_user'] = array(false, 'post', 'moderate');
}

/**
 * Give the admin some help
 *
 * Called by:
 * 		integrate_helpadmin
 */
function postalt_permission_help()
{
	loadLanguage('PostAsAnotherMember');
}

/**
 * Mention who made this thing
 *
 * Called by:
 * 		integrate_credits
 */
function postalt_credits()
{
	global $context;
	$context['copyrights']['mods'][] = 'Post As Another Member by Jon &quot;Sesquipedalian&quot; Stovell &copy; 2023';
}


/**
 * Sets and sanitizes $context['alternate_member_id']
 */
function postalt_get_alt_id()
{
	global $context;

	if (isset($_REQUEST['alternate_member_list']) && is_array($_REQUEST['alternate_member_list']) && count($_REQUEST['alternate_member_list']) == 1)
	{
		list($context['alternate_member_id']) = $_REQUEST['alternate_member_list'];
		$context['alternate_member_id'] = (int) $context['alternate_member_id'];
	}
	else
		$context['alternate_member_id'] = 0;
}